@@ -20,7 +20,7 @@ mod base;
 pub use base::{decode_function_data, encode_function_data, AbiError, BaseContract};
 
 mod call;
-pub use call::ContractError;
+pub use call::{ContractError, EthCall};
 
 mod factory;
 pub use factory::ContractFactory;
@@ -39,9 +39,7 @@ pub use multicall::Multicall;
 /// This module exposes low lever builder structures which are only consumed by the
 /// type-safe ABI bindings generators.
 pub mod builders {
-    pub use super::call::ContractCall;
-    pub use super::event::Event;
-    pub use super::factory::Deployer;
+    pub use super::{call::ContractCall, event::Event, factory::Deployer};
 }
 
 #[cfg(any(test, feature = "abigen"))]
@@ -50,8 +48,11 @@ pub use ethers_contract_abigen::Abigen;
 
 #[cfg(any(test, feature = "abigen"))]
 #[cfg_attr(docsrs, doc(cfg(feature = "abigen")))]
-pub use ethers_contract_derive::{abigen, EthAbiType, EthEvent};
+pub use ethers_contract_derive::{abigen, EthAbiType, EthCall, EthDisplay, EthEvent};
 
 // Hide the Lazy re-export, it's just for convenience
 #[doc(hidden)]
 pub use once_cell::sync::Lazy;
+
+#[cfg(feature = "eip712")]
+pub use ethers_derive_eip712::*;
