@@ -1,6 +1,6 @@
 use ethers_core::{
     abi::{Detokenize, Function, Token},
-    types::{Address, BlockNumber, Bytes, NameOrAddress, TxHash, U256},
+    types::{Address, BlockNumber, Bytes, Chain, NameOrAddress, TxHash, U256},
 };
 use ethers_providers::Middleware;
 
@@ -17,34 +17,20 @@ use multicall_contract::MulticallContract;
 /// A lazily computed hash map with the Ethereum network IDs as keys and the corresponding
 /// Multicall smart contract addresses as values
 pub static ADDRESS_BOOK: Lazy<HashMap<U256, Address>> = Lazy::new(|| {
-    let mut m = HashMap::new();
+    fn decode_address(input: &str) -> Address {
+        Address::from_str(input).expect("Decoding failed")
+    }
 
-    // mainnet
-    let addr =
-        Address::from_str("eefba1e63905ef1d7acba5a8513c70307c1ce441").expect("Decoding failed");
-    m.insert(U256::from(1u8), addr);
-
-    // rinkeby
-    let addr =
-        Address::from_str("42ad527de7d4e9d9d011ac45b31d8551f8fe9821").expect("Decoding failed");
-    m.insert(U256::from(4u8), addr);
-
-    // goerli
-    let addr =
-        Address::from_str("77dca2c955b15e9de4dbbcf1246b4b85b651e50e").expect("Decoding failed");
-    m.insert(U256::from(5u8), addr);
-
-    // kovan
-    let addr =
-        Address::from_str("2cc8688c5f75e365aaeeb4ea8d6a480405a48d2a").expect("Decoding failed");
-    m.insert(U256::from(42u8), addr);
-
-    // xdai
-    let addr =
-        Address::from_str("b5b692a88bdfc81ca69dcb1d924f59f0413a602a").expect("Decoding failed");
-    m.insert(U256::from(100u8), addr);
-
-    m
+    [
+        (Chain::Mainnet.into(), decode_address("eefba1e63905ef1d7acba5a8513c70307c1ce441")),
+        (Chain::Rinkeby.into(), decode_address("42ad527de7d4e9d9d011ac45b31d8551f8fe9821")),
+        (Chain::Goerli.into(), decode_address("77dca2c955b15e9de4dbbcf1246b4b85b651e50e")),
+        (Chain::Kovan.into(), decode_address("2cc8688c5f75e365aaeeb4ea8d6a480405a48d2a")),
+        (Chain::XDai.into(), decode_address("b5b692a88bdfc81ca69dcb1d924f59f0413a602a")),
+        (Chain::Polygon.into(), decode_address("11ce4B23bD875D7F5C6a31084f55fDe1e9A87507")),
+        (Chain::PolygonMumbai.into(), decode_address("08411ADd0b5AA8ee47563b146743C13b3556c9Cc")),
+    ]
+    .into()
 });
 
 /// A Multicall is an abstraction for sending batched calls/transactions to the Ethereum blockchain.
@@ -169,10 +155,8 @@ impl<M: Middleware> Multicall<M> {
         let address: Address = match address {
             Some(addr) => addr,
             None => {
-                let chain_id = client
-                    .get_chainid()
-                    .await
-                    .map_err(ContractError::MiddlewareError)?;
+                let chain_id =
+                    client.get_chainid().await.map_err(ContractError::MiddlewareError)?;
                 match ADDRESS_BOOK.get(&chain_id) {
                     Some(addr) => *addr,
                     None => panic!(
@@ -185,12 +169,7 @@ impl<M: Middleware> Multicall<M> {
         // Instantiate the multicall contract
         let contract = MulticallContract::new(address, client);
 
-        Ok(Self {
-            calls: vec![],
-            block: None,
-            contract,
-            legacy: false,
-        })
+        Ok(Self { calls: vec![], block: None, contract, legacy: false })
     }
 
     /// Makes a legacy transaction instead of an EIP-1559 one
@@ -212,17 +191,11 @@ impl<M: Middleware> Multicall<M> {
     /// If more than the maximum number of supported calls are added. The maximum
     /// limits is constrained due to tokenization/detokenization support for tuples
     pub fn add_call<D: Detokenize>(&mut self, call: ContractCall<M, D>) -> &mut Self {
-        if self.calls.len() >= 16 {
-            panic!("Cannot support more than {} calls", 16);
-        }
+        assert!(!(self.calls.len() >= 16), "Cannot support more than {} calls", 16);
 
         match (call.tx.to(), call.tx.data()) {
             (Some(NameOrAddress::Address(target)), Some(data)) => {
-                let call = Call {
-                    target: *target,
-                    data: data.clone(),
-                    function: call.function,
-                };
+                let call = Call { target: *target, data: data.clone(), function: call.function };
                 self.calls.push(call);
                 self
             }
@@ -372,11 +345,8 @@ impl<M: Middleware> Multicall<M> {
 
     fn as_contract_call(&self) -> ContractCall<M, (U256, Vec<Vec<u8>>)> {
         // Map the Multicall struct into appropriate types for `aggregate` function
-        let calls: Vec<(Address, Vec<u8>)> = self
-            .calls
-            .iter()
-            .map(|call| (call.target, call.data.to_vec()))
-            .collect();
+        let calls: Vec<(Address, Vec<u8>)> =
+            self.calls.iter().map(|call| (call.target, call.data.to_vec())).collect();
 
         // Construct the ContractCall for `aggregate` function to broadcast the transaction
         let mut contract_call = self.contract.aggregate(calls);
