@@ -1,19 +1,22 @@
-use ethers_contract::ContractFactory;
-use ethers_core::types::{Filter, ValueOrArray, H256};
-
+#![allow(unused)]
 mod common;
 pub use common::*;
+use ethers_contract::{abigen, ContractFactory, EthAbiType};
+use ethers_core::types::{Filter, ValueOrArray, H256};
 
 #[cfg(not(feature = "celo"))]
 mod eth_tests {
     use super::*;
     use ethers_contract::{LogMeta, Multicall};
     use ethers_core::{
-        types::{Address, BlockId, U256},
-        utils::Ganache,
+        types::{transaction::eip712::Eip712, Address, BlockId, I256, U256},
+        utils::{keccak256, Ganache},
     };
+    use ethers_derive_eip712::*;
+    use ethers_middleware::signer::SignerMiddleware;
     use ethers_providers::{Http, Middleware, PendingTransaction, Provider, StreamExt};
-    use std::{convert::TryFrom, sync::Arc};
+    use ethers_signers::{LocalWallet, Signer};
+    use std::{convert::TryFrom, sync::Arc, time::Duration};
 
     #[tokio::test]
     async fn deploy_and_call_contract() {
@@ -35,10 +38,10 @@ mod eth_tests {
         // `send` consumes the deployer so it must be cloned for later re-use
         // (practically it's not expected that you'll need to deploy multiple instances of
         // the _same_ deployer, so it's fine to clone here from a dev UX vs perf tradeoff)
-        let deployer = factory
-            .deploy("initial value".to_string())
-            .unwrap()
-            .legacy();
+        let deployer = factory.deploy("initial value".to_string()).unwrap().legacy();
+        // dry runs the deployment of the contract. takes the deployer by reference, no need to
+        // clone.
+        assert!(deployer.call().await.is_ok());
         let contract = deployer.clone().send().await.unwrap();
 
         let get_value = contract.method::<_, String>("getValue", ()).unwrap();
@@ -70,18 +73,10 @@ mod eth_tests {
         // (useful when interacting with multiple ERC20s for example)
         let contract2_addr = deployer.send().await.unwrap().address();
         let contract2 = contract.at(contract2_addr);
-        let init_value: String = contract2
-            .method::<_, String>("getValue", ())
-            .unwrap()
-            .call()
-            .await
-            .unwrap();
-        let init_address = contract2
-            .method::<_, Address>("lastSender", ())
-            .unwrap()
-            .call()
-            .await
-            .unwrap();
+        let init_value: String =
+            contract2.method::<_, String>("getValue", ()).unwrap().call().await.unwrap();
+        let init_address =
+            contract2.method::<_, Address>("lastSender", ()).unwrap().call().await.unwrap();
         assert_eq!(init_address, Address::zero());
         assert_eq!(init_value, "initial value");
 
@@ -107,10 +102,7 @@ mod eth_tests {
         let contract = deploy(client.clone(), abi, bytecode).await;
 
         // make a call with `client`
-        let func = contract
-            .method::<_, H256>("setValue", "hi".to_owned())
-            .unwrap()
-            .legacy();
+        let func = contract.method::<_, H256>("setValue", "hi".to_owned()).unwrap().legacy();
         let tx = func.send().await.unwrap();
         let _receipt = tx.await.unwrap();
 
@@ -181,13 +173,8 @@ mod eth_tests {
         let deployed_block = client.get_block_number().await.unwrap();
 
         // assert initial state
-        let value = contract
-            .method::<_, String>("getValue", ())
-            .unwrap()
-            .legacy()
-            .call()
-            .await
-            .unwrap();
+        let value =
+            contract.method::<_, String>("getValue", ()).unwrap().legacy().call().await.unwrap();
         assert_eq!(value, "initial value");
 
         // make a call with `client`
@@ -200,13 +187,8 @@ mod eth_tests {
             .unwrap();
 
         // assert new value
-        let value = contract
-            .method::<_, String>("getValue", ())
-            .unwrap()
-            .legacy()
-            .call()
-            .await
-            .unwrap();
+        let value =
+            contract.method::<_, String>("getValue", ()).unwrap().legacy().call().await.unwrap();
         assert_eq!(value, "hi");
 
         // assert previous value
@@ -247,39 +229,19 @@ mod eth_tests {
         let deployed_block = client.get_block_number().await.unwrap();
 
         // assert initial state
-        let value = contract
-            .method::<_, String>("getValue", ())
-            .unwrap()
-            .call()
-            .await
-            .unwrap();
+        let value = contract.method::<_, String>("getValue", ()).unwrap().call().await.unwrap();
         assert_eq!(value, "initial value");
 
         // make a call with `client`
-        let _tx_hash = *contract
-            .method::<_, H256>("setValue", "hi".to_owned())
-            .unwrap()
-            .send()
-            .await
-            .unwrap();
+        let _tx_hash =
+            *contract.method::<_, H256>("setValue", "hi".to_owned()).unwrap().send().await.unwrap();
 
         // assert new value
-        let value = contract
-            .method::<_, String>("getValue", ())
-            .unwrap()
-            .call()
-            .await
-            .unwrap();
+        let value = contract.method::<_, String>("getValue", ()).unwrap().call().await.unwrap();
         assert_eq!(value, "hi");
 
         // assert previous value using block hash
-        let hash = client
-            .get_block(deployed_block)
-            .await
-            .unwrap()
-            .unwrap()
-            .hash
-            .unwrap();
+        let hash = client.get_block(deployed_block).await.unwrap().unwrap().hash.unwrap();
         let value = contract
             .method::<_, String>("getValue", ())
             .unwrap()
@@ -318,10 +280,7 @@ mod eth_tests {
         // and we make a few calls
         let num = client.get_block_number().await.unwrap();
         for i in 0..num_calls {
-            let call = contract
-                .method::<_, H256>("setValue", i.to_string())
-                .unwrap()
-                .legacy();
+            let call = contract.method::<_, H256>("setValue", i.to_string()).unwrap().legacy();
             let pending_tx = call.send().await.unwrap();
             let _receipt = pending_tx.await.unwrap();
         }
@@ -335,13 +294,7 @@ mod eth_tests {
             assert_eq!(log.new_value, log2.new_value);
             assert_eq!(log.new_value, i.to_string());
             assert_eq!(meta.block_number, num + i + 1);
-            let hash = client
-                .get_block(num + i + 1)
-                .await
-                .unwrap()
-                .unwrap()
-                .hash
-                .unwrap();
+            let hash = client.get_block(num + i + 1).await.unwrap().unwrap().hash.unwrap();
             assert_eq!(meta.block_hash, hash);
         }
     }
@@ -355,24 +308,16 @@ mod eth_tests {
         let contract_2 = deploy(client.clone(), abi.clone(), bytecode).await;
 
         let ws = Provider::connect(ganache.ws_endpoint()).await.unwrap();
-        let filter = Filter::new().address(ValueOrArray::Array(vec![
-            contract_1.address(),
-            contract_2.address(),
-        ]));
+        let filter = Filter::new()
+            .address(ValueOrArray::Array(vec![contract_1.address(), contract_2.address()]));
         let mut stream = ws.subscribe_logs(&filter).await.unwrap();
 
         // and we make a few calls
-        let call = contract_1
-            .method::<_, H256>("setValue", "1".to_string())
-            .unwrap()
-            .legacy();
+        let call = contract_1.method::<_, H256>("setValue", "1".to_string()).unwrap().legacy();
         let pending_tx = call.send().await.unwrap();
         let _receipt = pending_tx.await.unwrap();
 
-        let call = contract_2
-            .method::<_, H256>("setValue", "2".to_string())
-            .unwrap()
-            .legacy();
+        let call = contract_2.method::<_, H256>("setValue", "2".to_string()).unwrap().legacy();
         let pending_tx = call.send().await.unwrap();
         let _receipt = pending_tx.await.unwrap();
 
@@ -410,12 +355,8 @@ mod eth_tests {
             .unwrap()
             .await
             .unwrap();
-        let value: String = contract
-            .method::<_, String>("getValue", ())
-            .unwrap()
-            .call()
-            .await
-            .unwrap();
+        let value: String =
+            contract.method::<_, String>("getValue", ()).unwrap().call().await.unwrap();
         assert_eq!(value, "hi");
     }
 
@@ -455,13 +396,8 @@ mod eth_tests {
         let not_so_simple_factory =
             ContractFactory::new(not_so_simple_abi, not_so_simple_bytecode, client3.clone());
 
-        let multicall_contract = multicall_factory
-            .deploy(())
-            .unwrap()
-            .legacy()
-            .send()
-            .await
-            .unwrap();
+        let multicall_contract =
+            multicall_factory.deploy(()).unwrap().legacy().send().await.unwrap();
         let addr = multicall_contract.address();
 
         let simple_contract = simple_factory
@@ -499,24 +435,15 @@ mod eth_tests {
 
         // get the calls for `value` and `last_sender` for both SimpleStorage contracts
         let value = simple_contract.method::<_, String>("getValue", ()).unwrap();
-        let value2 = not_so_simple_contract
-            .method::<_, (String, Address)>("getValues", ())
-            .unwrap();
-        let last_sender = simple_contract
-            .method::<_, Address>("lastSender", ())
-            .unwrap();
-        let last_sender2 = not_so_simple_contract
-            .method::<_, Address>("lastSender", ())
-            .unwrap();
+        let value2 =
+            not_so_simple_contract.method::<_, (String, Address)>("getValues", ()).unwrap();
+        let last_sender = simple_contract.method::<_, Address>("lastSender", ()).unwrap();
+        let last_sender2 = not_so_simple_contract.method::<_, Address>("lastSender", ()).unwrap();
 
         // initiate the Multicall instance and add calls one by one in builder style
         let mut multicall = Multicall::new(client4.clone(), Some(addr)).await.unwrap();
 
-        multicall
-            .add_call(value)
-            .add_call(value2)
-            .add_call(last_sender)
-            .add_call(last_sender2);
+        multicall.add_call(value).add_call(value2).add_call(last_sender).add_call(last_sender2);
 
         let return_data: (String, (String, Address), Address, Address) =
             multicall.call().await.unwrap();
@@ -542,16 +469,11 @@ mod eth_tests {
         // go. Now we will use the `.send()` functionality to broadcast a batch of transactions
         // in one go
         let mut multicall_send = multicall.clone();
-        multicall_send
-            .clear_calls()
-            .add_call(broadcast)
-            .add_call(broadcast2);
+        multicall_send.clear_calls().add_call(broadcast).add_call(broadcast2);
 
         // broadcast the transaction and wait for it to be mined
         let tx_hash = multicall_send.legacy().send().await.unwrap();
-        let _tx_receipt = PendingTransaction::new(tx_hash, client.provider())
-            .await
-            .unwrap();
+        let _tx_receipt = PendingTransaction::new(tx_hash, client.provider()).await.unwrap();
 
         // Do another multicall to check the updated return values
         // The `getValue` calls should return the last value we set in the batched broadcast
@@ -580,4 +502,145 @@ mod eth_tests {
         assert_eq!(balances.1, U256::from(100000000000000000000u128));
         assert_eq!(balances.2, U256::from(100000000000000000000u128));
     }
+
+    #[tokio::test]
+    async fn test_derive_eip712() {
+        // Generate Contract ABI Bindings
+        abigen!(
+            DeriveEip712Test,
+            "./ethers-contract/tests/solidity-contracts/derive_eip712_abi.json",
+            event_derives(serde::Deserialize, serde::Serialize)
+        );
+
+        // Create derived structs
+
+        #[derive(Debug, Clone, Eip712, EthAbiType)]
+        #[eip712(
+            name = "Eip712Test",
+            version = "1",
+            chain_id = 1,
+            verifying_contract = "0x0000000000000000000000000000000000000001",
+            salt = "eip712-test-75F0CCte"
+        )]
+        struct FooBar {
+            foo: I256,
+            bar: U256,
+            fizz: Vec<u8>,
+            buzz: [u8; 32],
+            far: String,
+            out: Address,
+        }
+
+        // get ABI and bytecode for the DeriveEip712Test contract
+        let (abi, bytecode) = compile_contract("DeriveEip712Test", "DeriveEip712Test.sol");
+
+        // launch ganache
+        let ganache = Ganache::new().spawn();
+
+        let wallet: LocalWallet = ganache.keys()[0].clone().into();
+
+        let provider = Provider::<Http>::try_from(ganache.endpoint())
+            .expect("failed to instantiate provider from ganache endpoint")
+            .interval(Duration::from_millis(10u64));
+
+        let client = SignerMiddleware::new(provider, wallet.clone());
+        let client = Arc::new(client);
+
+        let factory = ContractFactory::new(abi.clone(), bytecode.clone(), client.clone());
+
+        let contract = factory
+            .deploy(())
+            .expect("failed to deploy DeriveEip712Test contract")
+            .legacy()
+            .send()
+            .await
+            .expect("failed to instantiate factory for DeriveEip712 contract");
+
+        let addr = contract.address();
+
+        let contract = DeriveEip712Test::new(addr, client.clone());
+
+        let foo_bar = FooBar {
+            foo: I256::from(10),
+            bar: U256::from(20),
+            fizz: b"fizz".to_vec(),
+            buzz: keccak256("buzz"),
+            far: String::from("space"),
+            out: Address::from([0; 20]),
+        };
+
+        let derived_foo_bar = deriveeip712test_mod::FooBar {
+            foo: foo_bar.foo.clone(),
+            bar: foo_bar.bar.clone(),
+            fizz: foo_bar.fizz.clone(),
+            buzz: foo_bar.buzz.clone(),
+            far: foo_bar.far.clone(),
+            out: foo_bar.out.clone(),
+        };
+
+        let sig = wallet.sign_typed_data(&foo_bar).await.expect("failed to sign typed data");
+
+        let r = <[u8; 32]>::try_from(sig.r)
+            .expect("failed to parse 'r' value from signature into [u8; 32]");
+        let s = <[u8; 32]>::try_from(sig.s)
+            .expect("failed to parse 's' value from signature into [u8; 32]");
+        let v = u8::try_from(sig.v).expect("failed to parse 'v' value from signature into u8");
+
+        let domain_separator = contract
+            .domain_separator()
+            .call()
+            .await
+            .expect("failed to retrieve domain_separator from contract");
+        let type_hash =
+            contract.type_hash().call().await.expect("failed to retrieve type_hash from contract");
+        let struct_hash = contract
+            .struct_hash(derived_foo_bar.clone())
+            .call()
+            .await
+            .expect("failed to retrieve struct_hash from contract");
+        let encoded = contract
+            .encode_eip_712(derived_foo_bar.clone())
+            .call()
+            .await
+            .expect("failed to retrieve eip712 encoded hash from contract");
+        let verify = contract
+            .verify_foo_bar(wallet.address(), derived_foo_bar, r, s, v)
+            .call()
+            .await
+            .expect("failed to verify signed typed data eip712 payload");
+
+        assert_eq!(
+            domain_separator,
+            foo_bar
+                .domain()
+                .expect("failed to return domain_separator from Eip712 implemented struct")
+                .separator(),
+            "domain separator does not match contract domain separator!"
+        );
+
+        assert_eq!(
+            type_hash,
+            FooBar::type_hash().expect("failed to return type_hash from Eip712 implemented struct"),
+            "type hash does not match contract struct type hash!"
+        );
+
+        assert_eq!(
+            struct_hash,
+            foo_bar
+                .clone()
+                .struct_hash()
+                .expect("failed to return struct_hash from Eip712 implemented struct"),
+            "struct hash does not match contract struct hash!"
+        );
+
+        assert_eq!(
+            encoded,
+            foo_bar
+                .encode_eip712()
+                .expect("failed to return domain_separator from Eip712 implemented struct"),
+            "Encoded value does not match!"
+        );
+
+        assert_eq!(verify, true, "typed data signature failed!");
+    }
 }
